﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows;

namespace GlobalHookDemo
{
    class WordComparator : IComparer<WordEntry>
    {
        public int Compare(WordEntry w1, WordEntry w2)
        {       
            if (w1.Priority != w2.Priority)
                return w2.Priority.CompareTo(w1.Priority);
            if (w1.UseCount != w2.UseCount)
                return w2.UseCount.CompareTo(w1.UseCount);
            return w1.Word.CompareTo(w2.Word);

        }
    }

    class WordUnionComparator : IComparer<WordUnion>
    {
        public int Compare(WordUnion w1, WordUnion w2)
        {
            
            if (w1.useCount != w2.useCount)
                return w2.useCount.CompareTo(w1.useCount);
            return w1.word.CompareTo(w2.word);

        }
    }

    class DictionaryApi
    {
        public delegate void InitializeProgressBarDelegate();
        public event InitializeProgressBarDelegate InitializeProgressBarEvent;

        public delegate void ProgressBarMaximumDelegate(double max);
        public event ProgressBarMaximumDelegate ProgressBarMaximumEvent;

        public delegate void ProgressBarIncreaseDelegate();
        public event ProgressBarIncreaseDelegate ProgressBarIncreaseEvent;

        public delegate void FinishProgressBarDelegate();
        public event FinishProgressBarDelegate FinishProgressBarEvent;

        private static DictionaryApi instance;

        private DictionaryApi() { }

        bool znaleziono_zlepki = false;

        public static DictionaryApi Instance
        {
            get 
            {
                if (instance == null)
                {
                    instance = new DictionaryApi();
                }
                return instance;
            }
        }

        public SortedSet<WordEntry> words = new SortedSet<WordEntry>(new WordComparator()); 

        public void updateUsage(string slowo, string zlepek)
        {
            foreach (WordEntry word in words)
            {
                if (word.Word.Equals(slowo))
                {
                    word.UseCount++;
                    if (word.WordUnions.Count != 0 && zlepek!= null)
                    {
                        foreach (WordUnion union in word.WordUnions)
                        {
                            if(union.word.Equals(zlepek))
                            {
                                union.useCount++;
                            }
                        }
                    }
                }
                    
            }
            var copy = words.ToArray();
            WordUnionComparator comp = new WordUnionComparator();
            for (int i = 0; i < copy.Length; i++)
            {
                copy[i].WordUnions.Sort(comp);
            }
            this.words = new SortedSet<WordEntry>(copy, new WordComparator());
        }
        public IEnumerable<string> findWordsStartWith(string beginning)
        {
            var matchingList = new List<string>();
            //beginning = beginning.ToLower();


            if (beginning.Contains(' ') && beginning[0] != ' ')
            {
                znaleziono_zlepki = false;
                string[] slowa = beginning.Split(' ');
                int i = 0;
                int j = 0;
                foreach (WordEntry word in words)
                {
                    if (word.WordUnions.Count != 0 && word.Word.Equals(slowa[0]))
                    {
                        foreach (WordUnion union in word.WordUnions)
                        {
                            if (union.word.StartsWith(slowa[1]) || union.word.Equals(null))
                            {
                                matchingList.Add(union.word);
                                znaleziono_zlepki = true;
                            }
                            i++;
                        }
                        if (znaleziono_zlepki)
                            break;
                        if (i == word.WordUnions.Count)
                        {
                            foreach (WordEntry word1 in words)
                            {
                                if(word1.Word.StartsWith(slowa[1]))
                                    matchingList.Add(word1.Word);
                            }
                            break;
                        }
                    }
                    j++;
                }
                if (j == words.Count)
                {
                    foreach (WordEntry word1 in words)
                    {
                        if (word1.Word.StartsWith(slowa[1]))
                            matchingList.Add(word1.Word);
                    }
                }

            }
            else if (beginning.Equals(""))
            {
                foreach (WordEntry word in words)
                {
                    matchingList.Add(word.Word);
                }
            }
            else if (beginning[0] == ' ')
            {
                beginning = beginning.Trim();
                foreach (WordEntry word in words)
                {
                    if (word.Word.StartsWith(beginning))
                    {
                        matchingList.Add(word.Word);
                    }
                }

            }

            else
            {
                foreach (WordEntry word in words)
                {
                    if (word.Word.StartsWith(beginning))
                    {
                        matchingList.Add(word.Word);
                    }
                }
            }
               
            return matchingList;
        }

        public void RemoveWord(WordEntry word)
        {
            words.Remove(word);
        }

        public void analyze(string text, bool withEvents = true)
        {
            if(withEvents) InitializeProgressBarEvent();
            string[] words = Regex.Split(text, "[\r\n ]");
            if (withEvents) ProgressBarMaximumEvent(words.Length);
            for (int i = 0; i < words.Length; i++)
            {
                if (withEvents) ProgressBarIncreaseEvent();
                //addWord(words[i],null);
            }
            if (withEvents) FinishProgressBarEvent();
        }

        public void readDictionary(string text)
        {
            string[] words = Regex.Split(text, "[\n ]");
            for (int i = 0; i < words.Length; i++)
            {
                readEntry(words[i]);
            }
        }


        public void readEntry(string entry)
        {
            entry = entry.Trim();
            if(entry.Length == 0)
                return;
            string[] splitted = entry.Split(',');
            string word1 = null;
            int useCount1 = -1;
            try
            {
                int useCount = Convert.ToInt32(splitted[1]);
                int popularity = Convert.ToInt32(splitted[2]);

                if (splitted.Length > 3)
                {
                    List<WordUnion> wordUnions = new List<WordUnion>();
                    for (int i = 3; i < splitted.Length - 1; i = i + 2)
                    {
                        word1 = splitted[i];
                        useCount1 = Convert.ToInt32(splitted[i + 1]);
                        wordUnions.Add(new WordUnion(word1, useCount1));
                    }
                    words.Add(new WordEntry(splitted[0], useCount, popularity, wordUnions));
                }
                else
                {
                        words.Add(new WordEntry(splitted[0], useCount, popularity, new List<WordUnion>()));
                 
                }
            }
            catch { }
        }
        
        public WordEntry containsWord(SortedSet<WordEntry> words, string word){
            foreach(WordEntry w in words)
            {
                if (w.Word.StartsWith(word))
                    return w;
            }
            return null;
        }

        public void updateDictionary()
        {
            var copy = words.ToArray();
            this.words = new SortedSet<WordEntry>(copy, new WordComparator()); 
        }

        public void addWord(string word, string zlepek)
        {
            string pattern = "[,.\"':;/?()! ]+$";
            string replacement = "";
            Regex rgx = new Regex(pattern);
                word = rgx.Replace(word, replacement).Trim();
                if (zlepek != null)
                {
                    zlepek = rgx.Replace(zlepek, replacement).Trim();
                    if (zlepek.Length == 0)
                        zlepek = null;
                }
            if (word.Length == 0 || word.Equals("")) return;
            Boolean containsWord = false;
            Boolean ZlepekAsWord = false;
            foreach (WordEntry foundWord in words)
            {
                if(foundWord.Word.Equals(zlepek))
                {
                    ZlepekAsWord = true;
                }
                if (foundWord.Word.Equals(word))
                {
                    containsWord = true;
                    foundWord.UseCount++;
                    
                    if (foundWord.WordUnions != null && zlepek!=null)
                    {
                        bool containsUnion = false;
                        foreach (WordUnion union in foundWord.WordUnions)
                        {
                            if (zlepek.Equals(union.word)&&zlepek!=null)
                            {
                                union.useCount ++;
                                containsUnion = true;
                            }
                        }
                        
                        if (!containsUnion&&zlepek!=null)
                        {
                            List<WordUnion> wordUnions = new List<WordUnion>();
                            wordUnions = foundWord.WordUnions;
                            wordUnions.Add(new WordUnion(zlepek, 1));
                            foundWord.WordUnions = wordUnions;
                        }
                    }
                }
            }
            if (!containsWord)
            {
                List<WordUnion> wordUnions = new List<WordUnion>();
                if(zlepek!=null)
                    wordUnions.Add(new WordUnion(zlepek, 1));
                words.Add(new WordEntry(word,1,0,wordUnions));
            }
            if (!ZlepekAsWord&&zlepek!=null)
            {
                List<WordUnion> wordUnions = new List<WordUnion>();
                words.Add(new WordEntry(zlepek, 0, 0, wordUnions));
            }
        }

        public void SaveDictionary()
        {
            System.IO.StreamWriter file = new System.IO.StreamWriter("slownik.txt");
            
            foreach (WordEntry w in DictionaryApi.Instance.words)
            {
                if(w.WordUnions.Count == 0 )
                    file.WriteLine(w.Word + "," + w.UseCount + "," + w.Priority);
                else
                {
                    string line = w.Word + "," + w.UseCount + "," + w.Priority + ",";
                    int index = 0;
                    foreach(WordUnion union in w.WordUnions)
                    {
                        
                        if (index == w.WordUnions.Count-1)
                            line += union.word + "," + union.useCount;
                        else
                            line += union.word + "," + union.useCount + ",";
                        index++;

                    }
                    file.WriteLine(line);
                }
            }
            file.Close();
        }

        internal IEnumerable<string> findMostPopular()
        {
            var ret = new List<string>();
            foreach (var word in words)
            {
                if (word.WordUnions != null)
                {
                    word.WordUnions.Sort(new WordUnionComparator());
                    for (int i = 0 ; i< word.WordUnions.Count-1;i++)
                    ret.Add(word.WordUnions[i].word);
                }
                else
                    ret.Add(word.Word);
            }
            return ret;
        }
    }
}
