﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace GlobalHookDemo
{
    public partial class DictionaryControl : UserControl
    {
        private string currentString = "";

        public DictionaryControl()
        {
            InitializeComponent();
            UpdateSize();
            KeyboardHook.getInstance().Entered += new KeyboardHook.KeyEntered(DictionaryControl_Entered);
            listBox1.SelectedIndexChanged += new EventHandler(listBox1_SelectedIndexChanged);
            textBox1.TextChanged += new EventHandler(textBox1_TextChanged);
            Settings.Instance.SizeChanged += new EventHandler(Instance_SizeChanged);
        }

        void Instance_SizeChanged(object sender, EventArgs e)
        {
            UpdateSize();
        }

        void textBox1_TextChanged(object sender, EventArgs e)
        {
            listBox1.Items.Clear();
            var words = DictionaryApi.findWordsStartWith(currentString);
            if (words.Count() == 0)
            {
                words = DictionaryApi.findMostPopular();
            }
            listBox1.Items.AddRange(words.ToArray());
        }

        void listBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            string selectedText = listBox1.SelectedItem.ToString();
            string ToAdd = currentString.Equals("") ? selectedText : selectedText.ToLower().Replace(currentString.ToLower(), "");
            System.Diagnostics.Debug.WriteLine(ToAdd);

            SendKeys.SendWait(ToAdd.TrimEnd());
            textBox1.Text = this.currentString = "";
        }

        void DictionaryControl_Entered(object sender, KeyboardEventArgs e)
        {
            string txt = e.key;
            if (String.Compare(txt, "backspace") == 0) // backspace
            {
                if (currentString.Length != 0)
                    currentString = currentString.Substring(0, currentString.Length - 1);
            }
            else
            {
                currentString += txt;
            }
            if (txt[0] == '\n' || txt[0] == ' ')
            {
                DictionaryApi.addWord(currentString);
                this.currentString = "";
            }
            textBox1.Text = currentString;
        }

        void UpdateSize()
        {
            float size = Settings.Instance.Size;
            var font = new System.Drawing.Font(Font.Name, size,
                Font.Style, Font.Unit);
            textBox1.Font = listBox1.Font = font;
        }
    }
}
